unit ProblemDomain;

interface

uses
  Classes;

// Very simple unit to expose some business objects
// NOTE no framework for simplicity so all objects randomly initialise themselves!

type
  TPostcode = type String;

  TCustomerAttribute = (caLatePayer, caDiscounted, caNoDeliveryCharge);
  TCustomerAttributes = set of TCustomerAttribute;

  TPDObject = class
  end;

  TCustomer = class (TPDObject)
  private
    FName: String;
    FTelephoneNumber: String;
    FPostcode: TPostcode;
    FAddress: TStringList;
    FAttributes: TCustomerAttributes;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property Name: String read FName write FName;
    property Address: TStringList read FAddress;
    property Postcode: TPostcode read FPostcode write FPostcode;
    property TelephoneNumber: String read FTelephoneNumber write FTelephoneNumber;
    property Attributes: TCustomerAttributes read FAttributes write FAttributes;
  end;

  TStockItem = class (TPDObject)
  private
    FUnitPrice: Integer;
    FDescription: String;
  public
  published
    property Description: String read FDescription write FDescription;
    property UnitPrice: Integer read FUnitPrice write FUnitPrice;
  end;

  TOrder = class (TPDObject)
  private
    FQuantity: Integer;
    FPrice: Integer;
    FDescription: String;
    FCustomer: TCustomer;
    FStockItem: TStockItem;
    function GetCustomer: TCustomer;
    function GetStockItem: TStockItem;
    procedure SetCustomer(const Value: TCustomer);
    procedure SetStockItem(const Value: TStockItem);
  public
    destructor Destroy; override;
  published
    property Quantity: Integer read FQuantity write FQuantity;
    property Price: Integer read FPrice write FPrice;
    property Description: String read FDescription;

    property Customer: TCustomer read GetCustomer write SetCustomer;
    property StockItem: TStockItem read GetStockItem write SetStockItem;
  end;

implementation

uses
  SysUtils;

{ TCustomer }

constructor TCustomer.Create;
begin
  inherited;
  FAddress := TStringList.Create;
end;

destructor TCustomer.Destroy;
begin
  FAddress.Free;
  inherited;
end;

{ TStockItem }

{ TOrder }

destructor TOrder.Destroy;
begin
  FCustomer.Free;
  FStockItem.Free;
  inherited;
end;

function TOrder.GetCustomer: TCustomer;
begin
  if FCustomer = nil then FCustomer := TCustomer.Create;
  Result := FCustomer;
end;

function TOrder.GetStockItem: TStockItem;
begin
  if FStockItem = nil then FStockItem := TStockItem.Create;
  Result := FStockItem;
end;

procedure TOrder.SetCustomer(const Value: TCustomer);
begin
  FCustomer.Free;
  FCustomer := Value;
end;

procedure TOrder.SetStockItem(const Value: TStockItem);
begin
  FStockItem.Free;
  FStockItem := Value;
end;

end.
